@@ -51,14 +51,11 @@ void read_parms(void)
   char *p;
 
   /* Read global parameters */
-  if ((fp = fopen(parfile, "r")) == NULL) {
-    fprintf(stderr,
-            _("minicom: there is no global configuration file %s\n"), parfile);
-    fputs(_("Ask your sysadmin to create one (with minicom -s).\n"), stderr);
-    exit(1);
+  if ((fp = fopen(parfile, "r"))) {
+    readpars(fp, CONFIG_GLOBAL);
+    fclose(fp);
   }
-  readpars(fp, CONFIG_GLOBAL);
-  fclose(fp);
+
   /* Read personal parameters */
   if ((fp = fopen(pparfile, "r")) != NULL) {
     readpars(fp, CONFIG_PERSONAL);
@@ -817,7 +814,7 @@ static void doscrkeyb(void)
   char *macros_enabled        = _(" N - Macros enabled         :");
   char *character_conversion  = _(" O - Character conversion   :");
   char *add_linefeed          = _(" P - Add linefeed           :");
-  char *local_echo            = _(" Q - Local echo             :");
+  char *local_echo_str        = _(" Q - Local echo             :");
   char *question              = _("Change which setting?  (Esc to exit)");
 
   w = mc_wopen(15, miny, 69, maxy, BDOUBLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
@@ -846,7 +843,7 @@ static void doscrkeyb(void)
   mc_wprintf(w, "%s %s\n", character_conversion, P_CONVF);
 
   mc_wprintf(w, "%s %s\n", add_linefeed, _(P_ADDLINEFEED));
-  mc_wprintf(w, "%s %s\n", local_echo, _(P_LOCALECHO));
+  mc_wprintf(w, "%s %s\n", local_echo_str, _(P_LOCALECHO));
 
   mc_wredraw(w, 1);
 
@@ -1118,7 +1115,7 @@ static void doscrkeyb(void)
         break;
       case 'Q':
         psets(P_LOCALECHO, yesno(P_LOCALECHO[0] == 'N'));
-        mc_wlocate(w, mbslen (local_echo) + 1, 17);
+        mc_wlocate(w, mbslen (local_echo_str) + 1, 17);
         mc_wprintf(w, "%s", _(P_LOCALECHO));
         break;
      }
@@ -1245,6 +1242,8 @@ int dodflsave(void)
   }
   writepars(fp, dosetup);
   fclose(fp);
+  if (dosetup)
+    chmod(fname, (mode_t) 0644);
   werror(_("Configuration saved"));
 
   return domacsave() < 0 ? -1 : 0;
@@ -1567,9 +1566,9 @@ void domacros(void)
     mc_wlocate(w, 1, 15);
     mc_wputs(w, _("  \\u = username, \\p = password, \\\\ = \\, \\e = toggle echo, "));
     mc_wlocate(w, 1, 16);
-    mc_wputs(w, _("  \\l = toggle LF, \\bX1..Xn<sp> = change baud rate. Example: \"\\bfq \""));
+    mc_wputs(w, _("  \\l = toggle LF, \\bX1..Xn<sp> = change baud rate. Example: \"\\beq \""));
     mc_wlocate(w, 1, 17);
-    mc_wputs(w, _("  = 19200 8N1. (See the \"Comm Parameters\" menu for valid values of X.))"));
+    mc_wputs(w, _("  = 115200 8N1. (See the \"Comm Parameters\" menu for valid values of X.))"));
     if (clr) {
       mc_wlocate(w, 1, 12);
       mc_wprintf(w, "%s ", question);
@@ -1741,7 +1740,7 @@ void doconv(void)
         break;
       case 'C':
         prompt = _("Character to be edited: ");
-        mc_wprintf(w, prompt);
+        mc_wprintf(w, "%s", prompt);
         buf[0] = 0;
         i = -1;
         mc_wlocate(w, mbslen(prompt), ymax - 1);
