@@ -180,7 +180,7 @@ void mputs(const char *s, int how)
 {
   char c;
 
-  while(*s) {
+  while (*s) {
     if (*s == '^' && (*(s + 1))) {
       s++;
       if (*s == '^')
@@ -218,6 +218,9 @@ void mputs(const char *s, int how)
           toggle_local_echo();
           s++; /* again, move along. */
           continue;
+	case 'G': /* run a script */
+	  runscript(0, s + 1, "", "");
+	  return;
         default:
           s++;
           continue;
@@ -1119,7 +1122,7 @@ static void dedit(struct dialent *d)
   char *name		    = _(" A -  Name                :"),
        *number		    = _(" B -  Number              :"),
        *dial_string	    = _(" C -  Dial string #       :"),
-       *local_echo	    = _(" D -  Local echo          :"),
+       *local_echo_str	    = _(" D -  Local echo          :"),
        *script		    = _(" E -  Script              :"),
        *username	    = _(" F -  Username            :"),
        *password	    = _(" G -  Password            :"),
@@ -1134,7 +1137,7 @@ static void dedit(struct dialent *d)
   mc_wprintf(w, "%s %s\n", name, d->name);
   mc_wprintf(w, "%s %s\n", number, d->number);
   mc_wprintf(w, "%s %d\n", dial_string, d->dialtype + 1);
-  mc_wprintf(w, "%s %s\n", local_echo, _(yesno(d->flags & FL_ECHO)));
+  mc_wprintf(w, "%s %s\n", local_echo_str, _(yesno(d->flags & FL_ECHO)));
   mc_wprintf(w, "%s %s\n", script, d->script);
   mc_wprintf(w, "%s %s\n", username, d->username);
   mc_wprintf(w, "%s %s\n", password, d->password);
@@ -1179,7 +1182,7 @@ static void dedit(struct dialent *d)
         break;
       case 'D':
         d->flags ^= FL_ECHO;
-        mc_wlocate(w, mbslen (local_echo) + 1, 3);
+        mc_wlocate(w, mbslen (local_echo_str) + 1, 3);
         mc_wprintf(w, "%s", _(yesno(d->flags & FL_ECHO)));
         mc_wflush();
         break;
@@ -1254,7 +1257,7 @@ static const char *const what[] =
 /* Offsets of what[] entries from position_dialing_directory */
 #define DIALOPTS (sizeof(what) / sizeof(*what))
 #define DIAL_WIDTH 8 /* Width of one entry */
-static size_t what_lens[DIALOPTS]; /* Number of bytes for <= 7 characters */
+static int what_lens[DIALOPTS]; /* Number of bytes for <= 7 characters */
 /* Number of ' ' padding entries at left and right, left is >= 1 */
 static int what_padding[DIALOPTS][2];
 
