@@ -67,7 +67,7 @@ static const char *const what[] =
 #define WHAT_NR_OPTIONS (sizeof (what) / sizeof (*what))
 #define WHAT_WIDTH 8 /* Width of one entry */
 /* Number of bytes for <= 7 characters */
-static size_t what_lens[WHAT_NR_OPTIONS];
+static int what_lens[WHAT_NR_OPTIONS];
 /* Number of ' ' padding entries at left and right, left is >= 1 */
 static int what_padding[WHAT_NR_OPTIONS][2];
 static int dprev;
@@ -281,6 +281,8 @@ static int new_filedir(GETSDIR_ENTRY *o_dirdat, int flushit)
   int initial_y = (76 - (WHAT_NR_OPTIONS * WHAT_WIDTH >= 76
                    ? 74 : WHAT_NR_OPTIONS * WHAT_WIDTH)) / 2;
   size_t i;
+  int rval;
+  char * new_prev_dir;
 
   cur      =  0;
   ocur     =  0;
@@ -294,11 +296,6 @@ static int new_filedir(GETSDIR_ENTRY *o_dirdat, int flushit)
   dprev    = -1;
   tag_cnt  =  0;
 
-  /* got to do some error-checking here!!!  Maybe use mcd(), too! */
-  if (prev_dir != NULL)
-    free(prev_dir);
-  prev_dir = getcwd(NULL, BUFSIZ);
-
   /*
    * get last directory
    */
@@ -328,7 +325,30 @@ static int new_filedir(GETSDIR_ENTRY *o_dirdat, int flushit)
   if (strlen(work_dir) > 1 && work_dir[strlen(work_dir) - 1] == '/')
     work_dir[strlen(work_dir) - 1] = (char)0;
 
-  chdir(work_dir);
+  /* get the current working directory, which will become the prev_dir, on success */
+  new_prev_dir = getcwd(NULL, BUFSIZ);
+  if (new_prev_dir == NULL) {
+    return -1;
+  } 
+
+  rval = chdir(work_dir);
+  if (rval == 0) {
+    /* was able to change to new working directory */
+    free(prev_dir);
+    prev_dir = new_prev_dir;
+  }
+  else {
+    /* Could not change to the new working directory */
+    mc_wbell();
+    werror(
+        _("Could not change to directory %s (%s)"), 
+        work_dir,
+        strerror(errno));
+
+    /* restore the previous working directory */
+    free(work_dir);
+    work_dir = set_work_dir(new_prev_dir, strlen(new_prev_dir));
+  }
 
   /* All right, draw the file directory! */
 
@@ -434,7 +454,7 @@ static int new_filedir(GETSDIR_ENTRY *o_dirdat, int flushit)
     mc_wredraw(dsub, 1);
   }
 
-  return 0;
+  return rval;
 }
 
 
@@ -534,6 +554,8 @@ static char *concat_list(GETSDIR_ENTRY *d)
 {
   GETSDIR_ENTRY *my_d;
   int indxr, len;
+  int i;
+  char *j;
 
   my_d = d;
   for (indxr = nrents, len = 0; indxr; --indxr, ++my_d)
@@ -553,8 +575,18 @@ static char *concat_list(GETSDIR_ENTRY *d)
     for (indxr = nrents; indxr; --indxr, ++my_d)
       if (my_d->cflags & FL_TAG) {
         /* this could be *much* more efficient */
-        strcat(ret_buf, my_d->fname);
-        strcat(ret_buf, " ");
+        for (i = strlen(ret_buf), j = my_d->fname; *j; j++) {
+          if (*j == ' ') {
+            if ((ret_buf = (char*)realloc(ret_buf, ++len)) == NULL) {
+              file_tell(_("Too many files tagged - buffer would overflow"));
+              return(NULL);
+            }
+            ret_buf[i++] = '\\';
+          }
+          ret_buf[i++] = *j;
+        }
+        ret_buf[i++] = ' ';
+        ret_buf[i]   = '\0';
       }
 
     ret_buf[strlen(ret_buf) - 1] = (char)0;
