@@ -5,7 +5,7 @@
  *	Datatypes and constants for getsdir() - a function to get and
  *	return a sorted directory listing.
  *
- *	$Id: getsdir.h,v 1.3 2005/08/04 20:33:33 al-guest Exp $
+ *	$Id: getsdir.h,v 1.4 2007-10-10 20:18:20 al-guest Exp $
  *
  *	Copyright (c) 1998 by James S. Seymour (jseymour@jimsun.LinxNet.com)
  *
@@ -14,6 +14,10 @@
  *	as published by the Free Software Foundation; either version
  *	2 of the License, or (at your option) any later version.
  *
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
  *	Note: getsdir() uses "wildmat.c", which has different copyright
  *	and licensing conditions.  See the source, Luke.
  */
