@@ -151,10 +151,13 @@ void term_socket_close(void)
  *
  * \return -1 on error, 0 on success
  */
-int open_term(int doinit, int show_win_on_error)
+int open_term(int doinit, int show_win_on_error, int no_msgs)
 {
   struct stat stt;
-  char buf[128];
+  union {
+	char bytes[128];
+	int kermit;
+  } buf;
   int fd, n = 0;
   int pid;
 #ifdef HAVE_ERRNO_H
@@ -173,7 +176,7 @@ int open_term(int doinit, int show_win_on_error)
 #else /* SVR4_LOCKS */
     snprintf(lockfile, sizeof(lockfile),
                        "%s/LCK..%s",
-                       P_LOCK, mbasename(dial_tty, buf, sizeof(buf)));
+                       P_LOCK, mbasename(dial_tty, buf.bytes, sizeof(buf)));
 #endif /* SVR4_LOCKS */
 
   }
@@ -181,17 +184,17 @@ int open_term(int doinit, int show_win_on_error)
     lockfile[0] = 0;
 
   if (doinit > 0 && lockfile[0] && (fd = open(lockfile, O_RDONLY)) >= 0) {
-    n = read(fd, buf, 127);
+    n = read(fd, buf.bytes, 127);
     close(fd);
     if (n > 0) {
       pid = -1;
       if (n == 4)
         /* Kermit-style lockfile. */
-        pid = *(int *)buf;
+        pid = buf.kermit;
       else {
         /* Ascii lockfile. */
-        buf[n] = 0;
-        sscanf(buf, "%d", &pid);
+        buf.bytes[n] = 0;
+        sscanf(buf.bytes, "%d", &pid);
       }
       if (pid > 0 && kill((pid_t)pid, 0) < 0 &&
           errno == ESRCH) {
@@ -267,21 +270,25 @@ int open_term(int doinit, int show_win_on_error)
   alarm(0);
   signal(SIGALRM, SIG_IGN);
   if (portfd < 0 && !portfd_is_socket) {
-    if (doinit > 0) {
-      if (stdwin)
-	mc_wclose(stdwin, 1);
+    if (!no_msgs) {
+      if (doinit > 0) {
+	if (stdwin)
+	  mc_wclose(stdwin, 1);
 #ifdef HAVE_ERRNO_H
-      fprintf(stderr, _("minicom: cannot open %s: %s\n"),
-                      dial_tty, strerror(s_errno));
+	fprintf(stderr, _("minicom: cannot open %s: %s\n"),
+			dial_tty, strerror(s_errno));
 #else
-      fprintf(stderr, _("minicom: cannot open %s. Sorry.\n"), dial_tty);
+	fprintf(stderr, _("minicom: cannot open %s. Sorry.\n"), dial_tty);
 #endif
-      lockfile_remove();
-      return -1;
+        lockfile_remove();
+        return -1;
+      }
+
+      if (show_win_on_error)
+	werror(_("Cannot open %s!"), dial_tty);
     }
+
     lockfile_remove();
-    if (show_win_on_error)
-      werror(_("Cannot open %s!"), dial_tty);
     return -1;
   }
 
@@ -438,7 +445,7 @@ void time_status(void)
     mc_wprintf(st, " %12.12s ", P_HASDCD[0] == 'Y' ? _("Offline") : _("OFFLINE"));
   else
     mc_wprintf(st, " %s %02ld:%02ld", P_HASDCD[0] == 'Y' ? _("Online") : _("ONLINE"),
-            online / 3600, (online / 60) % 60);
+               online / 3600, (online / 60) % 60);
 
   ret_csr();
 }
@@ -453,25 +460,68 @@ void curs_status(void)
   ret_csr();
 }
 
+
+static char status_message[80];
+static int  status_display_msg_until;
+static int  status_message_showing;
+
+void status_set_display(const char *text, int duration_s)
+{
+  time_t t;
+  unsigned l;
+  strncpy(status_message, text, sizeof(status_message));
+  status_message[sizeof(status_message) - 1] = 0;
+  l = strlen(status_message);
+  for (; l < sizeof(status_message) - 1; ++l)
+    status_message[l] = ' ';
+
+  if (duration_s == 0)
+    duration_s = 2;
+
+  time(&t);
+  status_display_msg_until = duration_s + t;
+  status_message_showing = 1;
+}
+
+static void status_display_message(void)
+{
+  if (!st)
+    return;
+  mc_wlocate(st, 0, 0);
+  mc_wprintf(st, " %s", status_message);
+  ret_csr();
+}
+
 time_t old_online = -1;
 
 /*
  * Update the online time.
  */
-static void updtime(void)
+static void update_status_time(void)
 {
+  time_t now;
+  time(&now);
+
+  if (status_message_showing) {
+    if (now > status_display_msg_until) {
+      /* time over for status message, restore standard status line */
+      status_message_showing = 0;
+      if (st)
+	show_status();
+    } else
+      status_display_message();
+  }
 
   if (old_online == online)
     return;
-  if ((P_LOGCONN[0] == 'Y') && (old_online >= 0) && (online < 0)) {
+  if (P_LOGCONN[0] == 'Y' && old_online >= 0 && online < 0)
     do_log(_("Gone offline (%ld:%02ld:%02ld)"),
            old_online / 3600, (old_online / 60) % 60, old_online % 60);
-  }
+
   old_online = online;
-  if (st) {
+
+  if (!status_message_showing)
     time_status();
-    ret_csr();
-  }
   mc_wflush();
 }
 
@@ -506,7 +556,7 @@ void timer_update(void)
       time(&start);
       t1 = start;
       online = 0;
-      updtime();
+      update_status_time();
 #ifdef _DCDFLOW
       /* DCD has gotten high, we can turn on hw flow control */
       if (P_HASRTS[0] == 'Y')
@@ -525,11 +575,11 @@ void timer_update(void)
       /* First update the timer for call duration.. */
       time(&t1);
       online = t1 - start;
-      updtime();
+      update_status_time();
     }
     /* ..and THEN notify that we are now offline */
     online = -1;
-    updtime();
+    update_status_time();
   }
 
   /* Update online time */
@@ -537,7 +587,7 @@ void timer_update(void)
     time(&t1);
     online = t1 - start;
     if (online > (old_online + 59))
-      updtime();
+      update_status_time();
   }
 }
 
@@ -618,7 +668,7 @@ int do_terminal(void)
 
 dirty_goto:
   /* Show off or online time */
-  updtime();
+  update_status_time();
 
   /* If the status line was shown temporarily, delete it again. */
   if (tempst) {
@@ -663,7 +713,14 @@ dirty_goto:
 
     /* check if device is ok, if not, try to open it */
     if (!get_device_status(portfd_connected)) {
-      if (open_term(0, 0) < 0) {
+      /* Ok, it's gone, most probably someone unplugged the USB-serial, we
+       * need to free the FD so that a replug can get the same device
+       * filename, open it again and be back */
+      int reopen = portfd == -1;
+      close(portfd);
+      lockfile_remove();
+      portfd = -1;
+      if (open_term(reopen, reopen, 1) < 0) {
         if (!error_on_open_window)
           error_on_open_window = mc_tell(_("Cannot open %s!"), dial_tty);
       } else {
@@ -731,6 +788,9 @@ dirty_goto:
     if ((x & 2) == 2) {
       /* See which key was pressed. */
       c = keyboard(KGETKEY, 0);
+      if (c == EOF)
+        return EOF;
+
       if (c < 0) /* XXX - shouldn't happen */
         c += 256;
 
