@@ -461,7 +461,7 @@ static void scrollback(void)
   if (b_st->xs < 127)
     hline0[b_st->xs] = 0;
   hline = hline0;
-  mc_wprintf(b_st, hline);
+  mc_wprintf(b_st, "%s", hline);
   mc_wredraw(b_st, 1);
   mc_wflush();
 
@@ -514,7 +514,7 @@ static void scrollback(void)
         searchhist(b_us, look_for);
         /* must redraw status line... */
         mc_wlocate(b_st, 0, 0); /* move back to column 0! */
-        mc_wprintf(b_st, hline); /* and show the above-defined hline */
+        mc_wprintf(b_st, "%s", hline); /* and show the above-defined hline */
         mc_wredraw(b_st, 1); /* again... */
         /* highlight any matches */
         if (wcslen(look_for) > 1) {
@@ -723,7 +723,7 @@ static void scrollback(void)
           hline = hline0;
         }
         mc_wlocate(b_st, 0, 0);
-        mc_wprintf(b_st, hline);
+        mc_wprintf(b_st, "%s", hline);
         mc_wredraw(b_st, 1);
         if (citemode)
           mc_wlocate(b_us, 0, cite_y);
@@ -743,7 +743,7 @@ static void scrollback(void)
           break;
         }
         mc_wlocate(b_st, 0, 0);
-        mc_wprintf(b_st, hline);
+        mc_wprintf(b_st, "%s", hline);
         mc_wredraw(b_st, 1);
         mc_wdrawelm_inverse(b_us, cite_y, mc_getline(b_us, cite_ystart));
         mc_wlocate(b_us, 0, cite_y);
@@ -762,7 +762,7 @@ static void scrollback(void)
         }
         drawcite_whole(b_us, y, cite_ystart, cite_yend);
         mc_wlocate(b_st, 0, 0);
-        mc_wprintf(b_st, hline);
+        mc_wprintf(b_st, "%s", hline);
         mc_wredraw(b_st, 1);
         if (citemode)
           mc_wlocate(b_us, 0, cite_y);
@@ -847,6 +847,11 @@ void toggle_addlf(void)
   vt_set(addlf, -1, -1, -1, -1, -1, -1);
 }
 
+static void set_addlf(int val)
+{
+  vt_set(val, -1, -1, -1, -1, -1, -1);
+}
+
 /* Toggle local echo.  Can be called through the menu, or by a macro. */
 void toggle_local_echo(void)
 {
@@ -854,6 +859,10 @@ void toggle_local_echo(void)
   vt_set(-1, -1, -1, -1, local_echo, -1 ,-1);
 }
 
+static void set_local_echo(int val)
+{
+  vt_set(-1, -1, -1, -1, val, -1 ,-1);
+}
 
 /* -------------------------------------------- */
 
@@ -1272,14 +1281,14 @@ int main(int argc, char **argv)
 
   if (dial_tty == NULL) {
     if (!dosetup) {
-      while ((dial_tty = get_port(P_PORT)) != NULL && open_term(doinit, 1) < 0)
+      while ((dial_tty = get_port(P_PORT)) != NULL && open_term(doinit, 1, 0) < 0)
         ;
       if (dial_tty == NULL)
         exit(1);
     }
   }
   else {
-    if (!dosetup && open_term(doinit, 1) < 0)
+    if (!dosetup && open_term(doinit, 1, 0) < 0)
       exit(1);
   }
 
@@ -1296,7 +1305,7 @@ int main(int argc, char **argv)
       mc_wclose(stdwin, 1);
       exit(0);
     }
-    while ((dial_tty = get_port(P_PORT)) != NULL && open_term(doinit, 1) < 0)
+    while ((dial_tty = get_port(P_PORT)) != NULL && open_term(doinit, 1, 0) < 0)
       ;
     if (dial_tty == NULL)
       exit(1);
@@ -1363,9 +1372,9 @@ int main(int argc, char **argv)
 #if defined (__DATE__) && defined (__TIME__)
   mc_wprintf(us, "%s %s, %s.\r\n",_("Compiled on"), __DATE__,__TIME__);
 #endif
-  mc_wprintf(us, "Port %s\r\n", P_PORT);
+  mc_wprintf(us, "%s %s\r\n", _("Port"), P_PORT);
   if (using_iconv())
-    mc_wprintf(us, _("Using character set conversion\r\n"));
+    mc_wprintf(us, "%s\r\n", _("Using character set conversion"));
   mc_wprintf(us, _("\nPress %sZ for help on special keys%c\n\n"),esc_key(),'\r');
 
   readdialdir();
@@ -1376,20 +1385,23 @@ int main(int argc, char **argv)
   if (cmd_dial)
     dialone(cmd_dial);
 
+  set_local_echo(local_echo);
+  set_addlf(addlf);
+
   /* The main loop calls do_terminal and gets a function key back. */
   while (!quit) {
     c = do_terminal();
 dirty_goto:
     switch (c + 32 *(c >= 'A' && c <= 'Z')) {
       case 'a': /* Add line feed */
-	toggle_addlf();
+        toggle_addlf();
         s = addlf ?  _("Add linefeed ON") : _("Add linefeed OFF");
-        werror("%s", s);
+        status_set_display(s, 0);
         break;
       case 'e': /* Local echo on/off. */
-	toggle_local_echo();
+        toggle_local_echo();
         s = local_echo ?  _("Local echo ON") : _("Local echo OFF");
-        werror("%s", s);
+        status_set_display(s, 0);
         break;
       case 'z': /* Help */
         c = help();
@@ -1501,7 +1513,7 @@ dirty_goto:
         c = (!us->wrap);
         vt_set(addlf, c, docap, -1, -1, -1, -1);
         s = c ? _("Linewrap ON") : _("Linewrap OFF");
-        werror("%s", s);
+	status_set_display(s, 0);
         break;
       case 'o': /* Configure Minicom */
         (void) config(0);
@@ -1527,6 +1539,9 @@ dirty_goto:
       case 'y': /* Paste file */
 	paste_file();
 	break;
+      case EOF: /* Cannot read from stdin anymore, exit silently */
+        quit = NORESET;
+        break;
       default:
         break;
     }
