@@ -219,7 +219,7 @@ void vt_init(int type, int fg, int bg, int wrap, int add)
   mc_wresetregion(vt_win);
   vt_keypad = NORMAL;
   vt_cursor = NORMAL;
-  vt_echo = 0;
+  vt_echo = local_echo;
   vt_tabs[0] = 0x01010100;
   vt_tabs[1] =
   vt_tabs[2] =
@@ -565,7 +565,6 @@ static void state2(int c)
       vt_trans[0] = savetrans[0];
       vt_trans[1] = savetrans[1];
 #endif
-      break;
       vt_win->color = savecol; /* HACK should use mc_wsetfgcol etc */
       mc_wsetattr(vt_win, saveattr);
       mc_wlocate(vt_win, savex, savey);
@@ -615,18 +614,18 @@ static void state2(int c)
             mc_wsetfgcol(vt_win, vt_fg);
             mc_wsetbgcol(vt_win, vt_bg);
             break;
-          case 4:
-            attr |= XA_UNDERLINE;
-            break;
-          case 7:
-            attr |= XA_REVERSE;
-            break;
           case 1:
             attr |= XA_BOLD;
             break;
+          case 4:
+            attr |= XA_UNDERLINE;
+            break;
           case 5:
             attr |= XA_BLINK;
             break;
+          case 7:
+            attr |= XA_REVERSE;
+            break;
           case 22: /* Bold off */
             attr &= ~XA_BOLD;
             break;
@@ -1012,7 +1011,7 @@ void vt_out(int ch)
       if (!using_iconv()) {
       c = vt_inmap[c];    /* conversion 04.09.97 / jl */
 #if TRANSLATE
-      if (vt_type == VT100 && vt_asis == 0)
+      if (vt_type == VT100 && vt_trans[vt_charset] && vt_asis == 0)
         c = vt_trans[vt_charset][c];
 #endif
       }
